@@ -28,14 +28,14 @@ def flup_fastcgi(wsgi_app, bind, cwd=None, pidfile=None, logfile=None,
                     pid = None
 
             if pid is not None and  is_running(pid):
-                sys.exit('Already running (PID: %r)' % pid)
+                sys.exit('Already running (PID: {})'.format(pid))
             elif pid is not None:
                 logger.info('PID file was pointing to nonexistent process %r',
                             pid)
             else:
                 logger.info('PID file should contain a number')
         doublefork(pidfile, logfile, cwd, umask)
-    logger.info('Starting FastCGI server (flup), current working dir %r' % cwd)
+    logger.info('Starting FastCGI server (flup), current working dir %r', cwd)
     fcgi.WSGIServer(wsgi_app, bindAddress=bind, umask=umask,
                     debug=False, **params).run()
 
@@ -55,12 +55,13 @@ class Flup(Cli):
     '''
 
     def __init__(self, app, bind='', logfile=None, pidfile=None,
-                 cwd='.', umask=002, fastcgi_params=None):
+                 cwd='.', umask=2, fastcgi_params=None):
         self.app = app
         self.cwd = os.path.abspath(cwd)
         if ':' in bind:
             host, port = bind.split(':')
             port = int(port)
+            bind = (host, port)
         else:
             bind = os.path.abspath(bind or os.path.join(self.cwd, 'fcgi.sock'))
             safe_makedirs(bind)
@@ -90,15 +91,17 @@ class Flup(Cli):
         '''
         if self.pidfile:
             if not os.path.exists(self.pidfile):
-                sys.exit("Pidfile %r doesn't exist" % self.pidfile)
+                sys.exit("Pidfile {!r} doesn't exist".format(self.pidfile))
             with open(self.pidfile) as pidfile:
                 pid = int(pidfile.read())
             for sig in [signal.SIGINT, signal.SIGTERM, signal.SIGKILL]:
                 try:
                     if terminate(pid, sig, 3):
                         os.remove(self.pidfile)
-                        sys.exit()
-                except OSError, exc:
+                        # NOTE: we are not performing sys.exit here,
+                        # otherwise restart command will not work
+                        return
+                except OSError as exc:
                     if exc.errno != errno.ESRCH:
                         raise
                     elif sig == signal.SIGINT:
@@ -107,4 +110,5 @@ class Flup(Cli):
 
     def command_restart(self):
         self.command_stop()
-        self.command_start()
+        # restart is useless for non-daemon programs
+        self.command_start(daemonize=True)
