@@ -2,10 +2,10 @@
 
 import logging
 
+import six
 import cgi
 import re
-import widgets
-from . import convs
+from . import convs, widgets
 from ..utils import cached_property
 from collections import OrderedDict
 from .perms import FieldPerm
@@ -41,17 +41,19 @@ class BaseField(object):
     #: Short description of the field
     hint = None
 
+    help = ''
+
     def __init__(self, name, conv=None, parent=None, permissions=None, **kwargs):
         if self._obsolete & set(kwargs):
             raise TypeError(
-                    'Obsolete parameters are used: %s' %
-                        list(self._obsolete & set(kwargs)))
-        kwargs.update(dict(
+                    'Obsolete parameters are used: {}'.format(
+                                list(self._obsolete & set(kwargs))))
+        kwargs.update(
             parent=parent,
             name=name,
             conv=(conv or self.conv)(field=self),
             widget=(kwargs.get('widget') or self.widget)(field=self),
-        ))
+        )
         if permissions is not None:
             kwargs['perm_getter'] = FieldPerm(permissions)
         self._init_kwargs = kwargs
@@ -95,6 +97,10 @@ class BaseField(object):
         '''
         return self.form.errors.get(self.input_name)
 
+    @property
+    def help_message(self):
+        return self.help or self.form.get_help(self.input_name)
+
     @cached_property
     def clean_value(self):
         '''
@@ -113,7 +119,7 @@ class BaseField(object):
         if self.form.id:
             # We use template names in list to replace, so we must use it here to
             # insure unique IDs.
-            return '%s-%s' % (self.form.id, self.input_name)
+            return '{}-{}'.format(self.form.id, self.input_name)
         return self.input_name
 
     def from_python(self, value):
@@ -188,7 +194,7 @@ class Field(BaseField):
         if not self.multiple:
             values = [values]
         for value in values:
-            if not isinstance(value, basestring):
+            if not isinstance(value, six.string_types):
                 self.form.errors[self.input_name] = 'Given value has incompatible type'
                 return False
         return True
@@ -229,11 +235,11 @@ class FieldSet(AggregateField):
         if kwargs.get('parent'):
             conv = (conv or self.conv)(field=self)
             fields = [field(parent=self) for field in fields]
-        kwargs.update(dict(
+        kwargs.update(
             name=name,
             conv=conv,
             fields=fields,
-        ))
+        )
         BaseField.__init__(self, **kwargs)
 
     @property
@@ -262,7 +268,8 @@ class FieldSet(AggregateField):
     def set_raw_value(self, raw_data, value):
         # fills in raw_data multidict, resulting keys are field's absolute names
         assert isinstance(value, dict), \
-                'To set raw value on %r need dict, got %r' % (self.input_name, value)
+                'To set raw value on {!r} need dict, got {!r}'\
+                        .format(self.input_name, value)
         if not value:
             # Field set can be optional
             return
@@ -306,10 +313,10 @@ class FieldBlock(FieldSet):
     prefix = ''
 
     def __init__(self, title, fields=[], **kwargs):
-        kwargs.update(dict(
+        kwargs.update(
             title=title,
             fields=fields,
-        ))
+        )
         kwargs.setdefault('name', '') # XXX generate unique name
         FieldSet.__init__(self, **kwargs)
 
@@ -364,12 +371,12 @@ class FieldList(AggregateField):
         if parent:
             conv = (conv or self.conv)(field=self)
             field = field(parent=self)
-        kwargs.update(dict(
+        kwargs.update(
             parent=parent,
             name=name,
             conv=conv,
             field=field,
-        ))
+        )
         BaseField.__init__(self, **kwargs)
 
     @property
@@ -447,11 +454,10 @@ class FileField(Field):
         if not self.multiple:
             values = [values]
         for value in values:
-            if value and \
-               not isinstance(value, cgi.FieldStorage) and \
+            if not isinstance(value, cgi.FieldStorage) and \
+               value and \
                not hasattr(value, 'read'): # XXX is this right?
                 self.form.errors[self.input_name] = 'Given value is not file'
                 return False
         return True
 
-
